import datetime

from daydreamer import resources
from daydreamer.endpoints.baseendpoint import BaseEndpoint
from daydreamer.utils import clean_locals


class ReferenceData(BaseEndpoint):

    def get_sports(self, session=None):
        """
        Get all sports.
        
        :param session: requests session to be used.
        :return: list of sports.
        """
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v2/sports', session=session)
        return self.process_response(
            response.json().get('sports', []), resources.SportsDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_currencies(self, session=None):
        """
        Get currencies accepted.
        
        :param session: requests session to be used.
        :return: supported currencies.
        """
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v2/currencies', session=session)
        return self.process_response(
            response.json().get('currencies', []), resources.CurrencyDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_periods(self, sport_id, session=None):
        """
        Get periods for a given sport.
        
        :param sport_id: id of the sport to get periods breakdown for.
        :param session: requests session to be used.
        :return: sport periods breakdown.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/periods', params=params, session=session)
        return self.process_response(
            response.json().get('periods', []), resources.PeriodDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_leagues(self, sport_id, session=None):
        """
        Get leagues contained for a given sport.

        :param sport_id: id of the sport to get periods breakdown for.
        :param session: requests session to be used.
        :return: leagues for a given sport.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v2/leagues', params=params, session=session)
        return self.process_response(
            response.json().get('leagues', []), resources.LeagueDetails, date_time_sent, datetime.datetime.utcnow()
        )



import sys


_ver = sys.version_info

#: Python 2.x?
is_py2 = (_ver[0] == 2)

#: Python 3.x?
is_py3 = (_ver[0] == 3)


try:
    from builtins import FileNotFoundError
except ImportError:
    class FileNotFoundError(OSError):
        pass


if is_py2:
    basestring = basestring
    numeric_types = (int, long, float)
    integer_types = (int, long)
elif is_py3:
    basestring = (str, bytes)
    numeric_types = (int, float)
    integer_types = (int,)



import datetime

from daydreamer import resources
from daydreamer.enums import OddsFormat
from daydreamer.endpoints.baseendpoint import BaseEndpoint
from daydreamer.utils import clean_locals


class MarketData(BaseEndpoint):

    def get_fixtures(self, sport_id, league_ids=None, since=None, is_live=None, event_ids=None, session=None):
        """
        Get all unsettled fixtures for a given sport.
        
        :param sport_id: id of the sport.
        :param league_ids: list of league ids for given sport.
        :param since: Used to receive incremental updates. Use the value of last from previous fixtures response
        :param is_live: To retrieve ONLY live events set the value to islive =1. 
        :param event_ids: list of event ids to filter for.
        :param session: requests session to be used.
        :return: all non-settled events.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/fixtures', params=params, session=session)
        return self.process_response(
            response.json(), resources.FixtureDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_settled_fixtures(self, sport_id, league_ids=None, since=None, session=None):
        """
        Get fixtures settled within the last 24 hours.
        
        :param sport_id: id of the sport.
        :param league_ids: list of league ids for given sport.
        :param since: Used to receive incremental updates. Use the value of last from previous fixtures response
        :param session: requests session to be used.
        :return: settled fixtures.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/fixtures/settled', params=params, session=session)
        return self.process_response(
            response.json(), resources.SettledFixtureDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_special_fixtures(self, sport_id, league_ids=None, category=None, event_id=None,
                             special_id=None, since=None, session=None):
        """
        Get unsettled special fixtures for given sport.
        
        :param sport_id: id of the sport.
        :param league_ids: list of league ids for given sport.
        :param category: category of the special.
        :param event_id: id of the linked event.
        :param special_id: id of specific special to get.
        :param since: Used to receive incremental updates. Use the value of last from previous fixtures response
        :param session: requests session to be used.
        :return: all non-settled specials (contests)
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/fixtures/special', params=params, session=session)
        return self.process_response(
            response.json(), resources.SpecialFixtureDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_settled_special_fixtures(self, sport_id, league_ids=None, since=None, session=None):
        """
        Get special fixtures settled within the last 24 hours.

        :param sport_id: id of the sport.
        :param league_ids: list of league ids for given sport.
        :param since: Used to receive incremental updates. Use the value of last from previous fixtures response
        :param session: requests session to be used.
        :return: settled special fixtures.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/fixtures/special/settled', params=params, session=session)
        return self.process_response(
            response.json(), resources.SettledSpecialFixtureDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_inrunning(self, session=None):
        """
        Get all events that have a live status associated with them. 
        
        :param session: requests session to be used.
        :return: live events.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/inrunning', params=params, session=session)
        return self.process_response(
            response.json(), resources.InRunningDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_odds(self, sport_id, league_ids=None, event_ids=None, since=None, is_live=None,
                 odds_format=OddsFormat.Decimal.value, session=None):
        """
        Get straight and parlay odds for all non-settled events.
        
        :param sport_id: id of the sport.
        :param league_ids: list of league ids for given sport.
        :param since: Used to receive incremental updates. Use the value of last from previous fixtures response
        :param is_live: To retrieve ONLY live events set the value to islive =1. 
        :param event_ids: list of event ids to filter for.
        :param odds_format: Format in which we return the odds. 
        :param session: requests session to be used. 
        :return: straight and parlay odds for all non-settled events.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/odds', params=params, session=session)
        return self.process_response(
            response.json(), resources.OddsDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_special_odds(self, sport_id, league_ids=None, special_id=None, since=None,
                         odds_format=OddsFormat.Decimal.value, session=None):
        """
        Get special odds for all non-settled events.
        
        :param sport_id: id of the sport.
        :param league_ids: list of league ids for given sport.
        :param special_id: id of specific special to filter for.
        :param odds_format: Format in which we return the odds. 
        :param since: Used to receive incremental updates. Use the value of last from previous fixtures response
        :param session: requests session to be used. 
        :return: special odds for all non-settled events.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/odds/special', params=params, session=session)
        return self.process_response(
            response.json(), resources.OddsDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_line(self, sport_id, league_id, event_id, period_number, bet_type, team=None, side=None, handicap=None,
                 odds_format=OddsFormat.Decimal.value, session=None):
        """
        Get latest odds for a line.
        
        :param sport_id: id of the sport to which the line belongs.
        :param league_id: id of the league to which the line belongs.
        :param event_id: id of the event to which the line belongs.
        :param period_number: This represents the period of the match. 
        :param bet_type: bet_type of the line, see pinnacle.enums.BetType
        :param team: Chosen team type. This is needed only for SPREAD, MONEYLINE and TEAM_TOTAL_POINTS bet types
        :param side: Chosen side. This is needed only for TOTAL_POINTS and TEAM_TOTAL_POINTS bet type
        :param handicap: This is needed for SPREAD, TOTAL_POINTS and TEAM_TOTAL_POINTS bet type
        :param odds_format: Format in which we return the odds. 
        :param session: requests session to be used. 
        :return: latest line.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/line', params=params, session=session)
        return self.process_response(
            response.json(), resources.LineDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_special_lines(self, special_id, contestant_id, odds_format=OddsFormat.Decimal.value, session=None):
        """
        Get latest special line.
        
        :param special_id: id of the special.
        :param contestant_id: id of the participant in the special.
        :param odds_format: Format in which we return the odds.
        :param session: requests session to be used. 
        :return: latest special line.
        """
        params = clean_locals(locals())
        date_time_sent = datetime.datetime.utcnow()
        response = self.request("GET", method='v1/line/special', params=params, session=session)
        return self.process_response(
            response.json(), resources.SpecialLineDetails, date_time_sent, datetime.datetime.utcnow()
        )

    def get_teaser_groups(self):
        raise NotImplementedError

    def get_teaser_odds(self):
        raise NotImplementedError

    def get_teaser_lines(self):
        raise NotImplementedError

    def get_parlay_lines(self):
        raise NotImplementedError




import unittest
import unittest.mock as mock

from pinnacle.apiclient import APIClient
from pinnacle.endpoints.referencedata import ReferenceData


class ReferenceDataTest(unittest.TestCase):

    def setUp(self):
        client = APIClient('username', 'password')
        self.reference_data = ReferenceData(client)

    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.process_response')
    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.request', return_value=mock.Mock())
    def test_get_sports(self, mock_request, mock_process_response):
        self.reference_data.get_sports()

        mock_request.assert_called_once_with("GET", method='v2/sports', session=None)
        assert mock_process_response.call_count == 1

    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.process_response')
    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.request', return_value=mock.Mock())
    def test_get_currencies(self, mock_request, mock_process_response):
        self.reference_data.get_currencies()

        mock_request.assert_called_once_with("GET", method='v2/currencies', session=None)
        assert mock_process_response.call_count == 1

    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.process_response')
    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.request', return_value=mock.Mock())
    def test_get_periods(self, mock_request, mock_process_response):
        self.reference_data.get_periods(sport_id=1)

        mock_request.assert_called_once_with("GET", method='v1/periods', params={'sportId': 1}, session=None)
        assert mock_process_response.call_count == 1

    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.process_response')
    @mock.patch('pinnacle.endpoints.referencedata.ReferenceData.request', return_value=mock.Mock())
    def test_get_leagues(self, mock_request, mock_process_response):
        self.reference_data.get_leagues(sport_id=1)

        mock_request.assert_called_once_with("GET", method='v2/leagues', params={'sportId': 1}, session=None)
        assert mock_process_response.call_count == 1



from daydreamer.resources.baseresource import BaseResource

from daydreamer.resources.referencedataresources import (
    SportsDetails,
    CurrencyDetails,
    PeriodDetails,
    LeagueDetails,
)

from daydreamer.resources.accountresources import AccountDetails

from daydreamer.resources.marketdataresources import (
    FixtureDetails,
    SettledFixtureDetails,
    SpecialFixtureDetails,
    SettledSpecialFixtureDetails,
    OddsDetails,
    SpecialOddsDetails,
    LineDetails,
    SpecialLineDetails,
    InRunningDetails,
)

from daydreamer.resources.bettingresources import (
    BetDetails,
    PlaceBetDetails,
    PlaceSpecialBetDetails,
)



from daydreamer.resources.baseresource import BaseResource


class AccountDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'availableBalance': 'availableBalance',
            'outstandingTransactions': 'outstandingTransactions',
            'givenCredit': 'givenCredit',
            'currency': 'currency',
        }



from daydreamer.resources.baseresource import BaseResource


class SportsDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'id': 'id',
            'name': 'name',
            'hasOfferings': 'hasOfferings',
            'leagueSpecialsCount': 'leagueSpecialsCount',
            'eventSpecialsCount': 'eventSpecialsCount',
            'eventCount': 'eventCount',
        }


class CurrencyDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'code': 'code',
            'name': 'name',
            'rate': 'rate',
        }


class PeriodDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'number': 'number',
            'description': 'description',
            'shortDescription': 'shortDescription',
            'spreadDescription': 'spreadDescription',
            'moneylineDescription': 'moneylineDescription',
            'totalDescription': 'totalDescription',
            'team1TotalDescription': 'team1TotalDescription',
            'team2TotalDescription': 'team2TotalDescription',
            'spreadShortDescription': 'spreadShortDescription',
            'moneylineShortDescription': 'moneylineShortDescription',
            'totalShortDescription': 'totalShortDescription',
            'team1TotalShortDescription': 'team1TotalShortDescription',
            'team2TotalShortDescription': 'team2TotalShortDescription',
        }


class LeagueDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'id': 'id',
            'name': 'name',
            'homeTeamType': 'homeTeamType',
            'hasOfferings': 'hasOfferings',
            'allowRoundRobins': 'allowRoundRobins',
            'leagueSpecialsCount': 'leagueSpecialsCount',
            'eventSpecialsCount': 'eventSpecialsCount',
            'eventCount': 'eventCount',
        }



from daydreamer.resources.baseresource import BaseResource


class FixtureDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'sportId': 'sportId',
            'last': 'last',
            'leagues': 'leagues',
        }


class SettledFixtureDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'sportId': 'sportId',
            'last': 'last',
            'leagues': 'leagues',
        }


class SpecialFixtureDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'sportId': 'sportId',
            'last': 'last',
            'leagues': 'leagues',
        }


class SettledSpecialFixtureDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'sportId': 'sportId',
            'last': 'last',
            'leagues': 'leagues',
        }


class OddsDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'sportId': 'sportId',
            'last': 'last',
            'leagues': 'leagues',
        }


class SpecialOddsDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'sportId': 'sportId',
            'last': 'last',
            'leagues': 'leagues',
        }


class LineDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'status': 'status',
            'price': 'price',
            'lineId': 'lineId',
            'altLineId': 'altLineId',
            'team1Score': 'team1Score',
            'team2Score': 'team2Score',
            'team1RedCards': 'team1RedCards',
            'team2RedCards': 'team2RedCards',
            'maxRiskStake': 'maxRiskStake',
            'minRiskStake': 'minRiskStake',
            'maxWinStake': 'maxWinStake',
            'minWinStake': 'minWinStake',
            'effectiveAsOf': 'effectiveAsOf',
        }


class SpecialLineDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'status': 'status',
            'price': 'price',
            'lineId': 'lineId',
            'specialId': 'specialId',
            'contestantId': 'contestantId',
            'handicap': 'handicap',
            'maxRiskStake': 'maxRiskStake',
            'minRiskStake': 'minRiskStake',
            'maxWinStake': 'maxWinStake',
            'minWinStake': 'minWinStake',
        }


class InRunningDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'sports': 'sports',
        }




from daydreamer.endpoints.account import Account
from daydreamer.endpoints.betting import Betting
from daydreamer.endpoints.marketdata import MarketData
from daydreamer.endpoints.referencedata import ReferenceData



from daydreamer.resources.baseresource import BaseResource


class BetDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'betId': 'betId',
            'wagerNumber': 'wagerNumber',
            'placedAt': 'placedAt',
            'win': 'win',
            'risk': 'risk',
            'winLoss': 'winLoss',
            'betStatus': 'betStatus',
            'betType': 'betType',
            'sportId': 'sportId',
            'leagueId': 'leagueId',
            'eventId': 'eventId',
            'handicap': 'handicap',
            'price': 'price',
            'teamName': 'teamName',
            'side': 'side',
            'oddsFormat': 'oddsFormat',
            'customerCommission': 'customerCommission',
            'pitcher1': 'pitcher1',
            'pitcher2': 'pitcher2',
            'pitcher1MustStart': 'pitcher1MustStart',
            'pitcher2MustStart': 'pitcher2MustStart',
            'team1': 'team1',
            'team2': 'team2',
            'isLive': 'isLive',
            'periodNumber': 'periodNumber',
            'team1Score': 'team1Score',
            'team2Score': 'team2Score',
            'ftTeam1Score': 'ftTeam1Score',
            'ftTeam2Score': 'ftTeam2Score',
            'pTeam1Score': 'pTeam1Score',
            'pTeam2Score': 'pTeam2Score',
            'cancellationReason': 'cancellationReason',
            'updateSequence': 'updateSequence',
        }


class PlaceBetDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'status': 'status',
            'errorCode': 'errorCode',
            'betId': 'betId',
            'uniqueRequestId': 'uniqueRequestId',
            'betterLineWasAccepted': 'betterLineWasAccepted',
            'price': 'price',
        }


class PlaceSpecialBetDetails(BaseResource):
    class Meta(BaseResource.Meta):
        attributes = {
            'status': 'status',
            'errorCode': 'errorCode',
            'betId': 'betId',
            'uniqueRequestId': 'uniqueRequestId',
            'betterLineWasAccepted': 'betterLineWasAccepted',
        }




from .exceptions import StatusCodeError


def check_status_code(response, codes=None):
    """
    Checks response.status_code is in codes.
    :param requests.request response: Requests response
    :param list codes: List of accepted codes or callable
    :raises: StatusCodeError if code invalid
    """
    codes = codes or [200]
    if response.status_code not in codes:
        raise StatusCodeError(response.status_code)


def clean_locals(data):
    """
    Clean up locals dict, remove empty and self/session/params params
    and convert to camelCase.
    :param {} data: locals dicts from a function.
    :returns: dict
    """
    return {
        to_camel_case(k): v for k, v in data.items() if v is not None and k not in
        ['self', 'session']
    }


def to_camel_case(snake_str):
    """
    Converts snake_string to camelCase
    :param str snake_str:
    :returns: str
    """
    components = snake_str.split('_')
    return components[0] + "".join(x.title() for x in components[1:])



import unittest

from pinnacle.apiclient import APIClient
from pinnacle.endpoints import Betting, Account, MarketData, ReferenceData


class APIClientTest(unittest.TestCase):

    def test_apiclient_init(self):
        client = APIClient('username', 'password')
        assert str(client) == 'APIClient'
        assert repr(client) == '<APIClient [username]>'
        assert isinstance(client.betting, Betting)
        assert isinstance(client.account, Account)
        assert isinstance(client.market_data, MarketData)
        assert isinstance(client.reference_data, ReferenceData)
